@@ -1,15 +1,15 @@
 ncal <- function(...) UseMethod("ncal") 
 
-# plot=TRUE; auto.layout=TRUE; plot.se.profile=TRUE; force.fit=FALSE; test.LOD=FALSE; find.LOD=FALSE; find.best.dilution=FALSE; return.fits=FALSE; grid.len=NULL; unk.replicate=NULL; verbose=FALSE; robust="mean"; weighting=F; incr=NULL; fit.4pl=FALSE; lod.ci=95; auto.detect.incr=T; plot.log="x"; control.jags=list(n.iter=1e5, jags.seed=1, n.thin=1e1, keep.jags.samples=FALSE)
+# plot=TRUE; auto.layout=TRUE; plot.se.profile=TRUE; force.fit=FALSE; test.LOD=FALSE; find.LOD=FALSE; find.best.dilution=FALSE; return.fits=FALSE; grid.len=NULL; unk.replicate=NULL; verbose=FALSE; robust="mean"; weighting=F; ; fit.4pl=FALSE; lod.ci=95; plot.log="x"; control.jags=list(n.iter=1e5, jags.seed=1, n.thin=1e1, keep.jags.samples=FALSE)
 ncal.formula = function (formula, data, 
     bcrm.fit=FALSE, bcrm.model="t4", robust="mean", 
     weighting=FALSE, pow.weight=1,
-    auto.detect.incr=TRUE, incr=NULL, 
     force.fit=TRUE, fit.4pl=FALSE, return.fits=FALSE, 
     plot=TRUE, auto.layout=TRUE, plot.se.profile=TRUE, plot.log="x", 
     test.LOD=FALSE, find.LOD=FALSE, find.LOQ=FALSE, grid.len=50, lod.ci=95,
     unk.replicate=NULL, find.best.dilution=FALSE, verbose=FALSE,     
-    control.jags=list(n.iter=1e5, jags.seed=1, n.thin=NULL, keep.jags.samples=FALSE),
+    control.jags=list(n.iter=1e5, jags.seed=1, n.thin=NULL, keep.jags.samples=FALSE, n.adapt=1e3), 
+    cex=.5,
 ...)
 {
     
@@ -18,13 +18,15 @@ ncal.formula = function (formula, data,
     if (is.null(control.jags$jags.seed)) control.jags$jags.seed=1
     if (is.null(control.jags$n.thin)) control.jags$n.thin=NULL
     if (is.null(control.jags$keep.jags.samples)) control.jags$keep.jags.samples=T
+    if (is.null(control.jags$n.adapt)) control.jags$n.adapt=1e3
     
+    # for backward compatibility with the rumi function call
     if (is.null(formula)) {
         formula=log(fi) ~ expected_conc
     }
     
-    if (is.null(data$starting_conc) & is.null(data$expected_conc)) stop("ERROR: starting_conc and expected_conc missing from data\n\n")
-    if (is.null(data$expected_conc)) data$expected_conc=data$starting_conc/data$dilution
+#    if (is.null(data$starting_conc) & is.null(data$expected_conc)) stop("ERROR: starting_conc and expected_conc missing from data\n\n")
+#    if (is.null(data$expected_conc)) data$expected_conc=data$starting_conc/data$dilution
     
     if (any(is.na(model.matrix(formula, data[data$well_role=="Standard",])[,2]))) {
         stop("some predictors are NA")
@@ -40,16 +42,21 @@ ncal.formula = function (formula, data,
         if (any(data[[outcome.coln]]<=0)) stop(outcome.coln%+%" var cannot be 0 or less")
     }
                     
-    if (!is.null(incr)) auto.detect.incr=TRUE
-    
-    # convert bead_type to analyte
+    # convert bead_type to analyte, backward compatibility
     if (is.null(data$analyte) & !is.null(data$bead_type)) data$analyte=data$bead_type
     
     # checking the data
-    if (is.null(data$sample_id)) {cat("ERROR: sample_id missing from data\n\n"); stop()}
-    if (is.null(data$assay_id)) {cat("ERROR: assay_id missing from data\n\n"); stop()}
-    if (is.null(data$analyte)) {cat("ERROR: analyte missing from data\n\n"); stop()}
-    if (is.null(data$well_role)) {cat("ERROR: well_role missing from data\n\n"); stop()}
+    if (is.null(data$assay_id)) stop("assay_id missing from data")
+    if (is.null(data$analyte)) stop("analyte missing from data")
+    if (is.null(data$well_role)) {
+        if (!is.null(data$sample_id)) stop("well_role missing from data")
+        # otherwise, assume all are standard wells
+        #warning("no well_role column, assume all samples are standard samples")
+        data$well_role="Standard"
+    }
+    if (is.null(data$sample_id) & any(data$well_role!="Standard")) {
+        stop("sample_id missing from data with not just standard samples\n\n")
+    }
     
     if (is.null(data[[outcome.coln]])) {cat("ERROR: y. missing from data\n\n"); stop()}
     if (!all(is.numeric(data[[outcome.coln]]))) stop("ERROR: y. column not numeric\n\n")
@@ -74,7 +81,9 @@ ncal.formula = function (formula, data,
         if (bcrm.fit) {
             dat.std.bcrm=data[data$well_role=="Standard" & data$analyte==a,] 
             if (nrow(dat.std.bcrm)==0) next    
-            fits = bcrm (formula, data=dat.std.bcrm, error.model="gh_"%+%bcrm.model, informative.prior=TRUE, keep.data=TRUE, n.iter=control.jags$n.iter, jags.seed=control.jags$jags.seed, n.thin=control.jags$n.thin, keep.jags.samples=control.jags$keep.jags.samples, T.init=NULL, prior.sensitivity=NULL, verbose=verbose)
+            fits = bcrm (formula, data=dat.std.bcrm, error.model="gh_"%+%bcrm.model, informative.prior=TRUE, keep.data=TRUE, fit.4pl=fit.4pl,
+                n.iter=control.jags$n.iter, jags.seed=control.jags$jags.seed, n.thin=control.jags$n.thin, keep.jags.samples=control.jags$keep.jags.samples, 
+                T.init=NULL, prior.sensitivity=NULL, n.adapt=control.jags$n.adapt, verbose=verbose)
         } 
         
         for (p in assays) {
@@ -101,7 +110,7 @@ ncal.formula = function (formula, data,
             } else {
                 fit = drm.fit(formula, data = dat.std, robust=robust, weighting=weighting, force.fit=force.fit, fit.4pl=fit.4pl, verbose=verbose, pow.weight=pow.weight)
             }
-            if (return.fits) fits[[p%+%a]]=fit
+            if (return.fits & !bcrm.fit) fits[[p%+%a]]=fit
             if (verbose) print("debug 100")
             
             out.ana=data.frame()
@@ -115,7 +124,7 @@ ncal.formula = function (formula, data,
                     # this pch is not used by plot.drc, nonetheless, keep it here for now
                     if (!is.null(dat.std$replicate)) pch=ifelse(dat.std$replicate==1, 1, 19) else pch=1
                     suppressWarnings(
-                        plot(fit, type="all", main=p%+%", "%+%a, cex=.5, xlim=exp(c(std.low, std.high)), pch=pch, log=plot.log, xlab=predictor.coln, 
+                        plot(fit, type="all", main=p%+%", "%+%a, cex=cex, xlim=exp(c(std.low, std.high)), pch=pch, log=plot.log, xlab=predictor.coln, 
                             ylab=ifelse(log.transform,"log(","")%+%outcome.coln%+%ifelse(log.transform,")",""))
                     )
                 }
@@ -130,20 +139,20 @@ ncal.formula = function (formula, data,
                 if (verbose & bad.se) print("bad.se")
                 if (bad.se & plot.se.profile) empty.plot()
                 
-                if (auto.detect.incr) { 
-                    coef.tmp=coef(fit,type="classical")
-                    incr = coef.tmp[startsWith(names(coef.tmp),"b")]<0 
-                }
+                coef.tmp=coef(fit,type="classical")
+                incr. = coef.tmp[startsWith(names(coef.tmp),"b")]<0 
                 if (verbose) print("debug 500")
     
                 # estimate unknown concentrations
                 dat.unk=dat.a.p[dat.a.p$well_role!="Standard",]
+                if (nrow(dat.unk)>0 & is.null(dat.unk$dilution)) dat.unk$dilution=1
                 sam = unique(dat.unk$sample_id)
                 #if (!is.null(sam)) sam=sort(sam) # do not think it is a good idea anymore. v15-16
                 for (s in sam) {
                     
                     if (verbose) myprint(s)                    
                     dil=sort(unlist(unique(dat.unk[dat.unk$sample_id==s, "dilution"])))
+                    
                     out.s=data.frame()
                     for (dil. in dil) {
                         if (verbose) myprint(dil.)
@@ -152,8 +161,9 @@ ncal.formula = function (formula, data,
                         if (is.null(unk.replicate)) unk.replicate=nrow(dat.unk.s.d)
                         y.=dat.unk.s.d[[outcome.coln]]
                         if (log.transform) y.=log(y.)
-                        estimated = getConc(fit, mean(y.), n.replicate=length(y.), x.range=exp(c(std.low,std.high)), y.range=c(y.low,y.high), fit.4pl = fit.4pl, incr=incr, verbose=verbose)
+                        estimated = getConc(fit, mean(y.), n.replicate=length(y.), x.range=exp(c(std.low,std.high)), y.range=c(y.low,y.high), verbose=verbose)
                         estimated = unname(estimated)
+                        if (verbose) print(estimated)
     
                         if (!bad.se) {
     
@@ -168,13 +178,14 @@ ncal.formula = function (formula, data,
                                 # set std err to Inf for those touched
                                 if(test.1.not.rejected | test.2.not.rejected) estimated[2] = Inf
                             }
-    
+                            
                             out.s=rbind (out.s, data.frame (c(dat.unk.s.d[1,], "est.log.conc"=estimated[1]+log(dil.), "se"=estimated[2], 
-                                "est.conc"=exp(estimated[1]+log(dil.)), "lb.conc"=exp(estimated[1]+log(dil.)-dd.95*estimated[2]), "ub.conc"=exp(estimated[1]+log(dil.)+dd.95*estimated[2])  )))
+                                "est.conc"=exp(estimated[1]+log(dil.)), "lb.conc"=exp(estimated[1]+log(dil.)-dd.95*estimated[2]), "ub.conc"=exp(estimated[1]+log(dil.)+dd.95*estimated[2])  ),
+                                stringsAsFactors =FALSE))
                         
                         } else {
                             out.s=rbind (out.s, data.frame (c(dat.unk.s.d[1,], "est.log.conc"=estimated[1]+log(dil.), "se"=NA, 
-                                "est.conc"=exp(estimated[1]+log(dil.)), "lb.conc"=NA, "ub.conc"=NA  )))
+                                "est.conc"=exp(estimated[1]+log(dil.)), "lb.conc"=NA, "ub.conc"=NA  ), stringsAsFactors =FALSE))
                         }
                         # let outcome.coln be the average
                         out.s[nrow(out.s),outcome.coln]=ifelse(log.transform, exp(mean(y.)), mean(y.))
@@ -198,7 +209,7 @@ ncal.formula = function (formula, data,
                     fit.low=FivePL.t(std.low, coef(fit))
                     fit.high=FivePL.t(std.high, coef(fit))
                     tmp.flag = fit.low>fit.high
-                    if (!incr) tmp.flag=!tmp.flag
+                    if (!incr.) tmp.flag=!tmp.flag
                     if (tmp.flag) {
                         if (verbose) print("out of here")
                         next
@@ -207,7 +218,7 @@ ncal.formula = function (formula, data,
                     if(verbose) print("nCal 620")
                     # low end
                     x.hat.low=getConc(fit, seq(fit.low, mid.log.fi, length=grid.len+1)[-1], n.replicate=rep(unk.replicate,grid.len), x.range=exp(c(std.low,std.high)), 
-                        y.range=c(y.low,y.high), fit.4pl = fit.4pl, incr=incr, verbose=verbose)[,c(1,2,6,7,8,4,5)]
+                        y.range=c(y.low,y.high), verbose=verbose)[,c(1,2,6,7,8,4,5)]
                     same.as.low = x.hat.low[,1] - dd*x.hat.low[,2] < std.low
                     #same.as.low = exp(x.hat.low[,1]) - dd * exp(x.hat.low[,1]) * x.hat.low[,2] < exp(std.low)
                     same.as.low.rle=rle(same.as.low)
@@ -216,7 +227,7 @@ ncal.formula = function (formula, data,
                     if(verbose) print("nCal 640")
                     # high end
                     x.hat.high=getConc(fit, seq(fit.high, mid.log.fi, length=grid.len+1)[-1], n.replicate=rep(unk.replicate,grid.len), x.range=exp(c(std.low,std.high)), 
-                        y.range=c(y.low,y.high), fit.4pl = fit.4pl, incr=incr)[,c(1,2,6,7,8,4,5)]
+                        y.range=c(y.low,y.high))[,c(1,2,6,7,8,4,5)]
                     same.as.high = x.hat.high[,1] + dd * x.hat.high[,2] > std.high
                     #same.as.high = exp(x.hat.high[,1]) + dd * exp(x.hat.high[,1]) * x.hat.high[,2] > exp(std.high)
                     same.as.high.rle=rle(same.as.high)
@@ -278,13 +289,15 @@ ncal.formula = function (formula, data,
                     if (plot.se.profile) {
                         
                         # fitted curves, with predicted conc
-                        suppressWarnings( plot(fit, type="all", main=p%+%", "%+%a, cex=.5, xlim=exp(c(std.low,std.high)),pch=pch,log=plot.log,xlab=predictor.coln,
+                        suppressWarnings( plot(fit, type="all", main=p%+%", "%+%a, cex=cex, xlim=exp(c(std.low,std.high)),pch=pch,log=plot.log,xlab=predictor.coln,
                                                     ylab=ifelse(log.transform,"log(","")%+%outcome.coln%+%ifelse(log.transform,")",""), ...) )
                         #abline(v=c(exp(x.low), exp(x.high)), col=1)
                         abline(v=LOQ[nrow(LOQ),3:4], col="gray")
                         #abline(v=LOQ.30[nrow(LOQ.30),3:4], col=3)
-                        if(log.transform) y.tmp=log(out.ana[[outcome.coln]]) else y.tmp=out.ana[[outcome.coln]]
-                        if (nrow(out.ana)>0) points(out.ana$est.conc/out.ana$dilution, y.tmp, pch="*", col=2)
+                        if (nrow(out.ana)>0) {
+                            if(log.transform) y.tmp=log(out.ana[[outcome.coln]]) else y.tmp=out.ana[[outcome.coln]]
+                            points(out.ana$est.conc/out.ana$dilution, y.tmp, pch="*", col=2)
+                        }
                         mylegend(legend=c("LOQ"), col=c("gray",2,1), lty=c(1,0), pch=c("","*"), x=2)
                         #mylegend(legend=c("LOQ","estimate of unknowns"), col=c("gray",2,1), lty=c(1,0), pch=c("","*"), x=2)
                         
@@ -360,35 +373,34 @@ ncal.formula = function (formula, data,
 
 ncal.character = function (file, is.luminex.xls, formula, bcrm.fit, ...) {
     if (is.luminex.xls) dat=read.luminex.xls(file) else dat=read.csv(file, as.is=TRUE)
-    ncal.formula(formula, dat, bcrm.fit=bcrm.fit)
+    ncal.formula(formula, dat, bcrm.fit=bcrm.fit, ...)
 }
 
 rumi = function(data, ...) ncal.formula(formula=log(fi)~expected_conc, data, ...)
     
 
-# fit is a drc object or a bcrm object, y is outcome, n.replicate is the number of replicates that give rise to y
-# vectorized for y
-# n.replicate=1; fit.4pl=FALSE; incr=TRUE; check.out.of.range=F; x.range=NULL; y.range=NULL; verbose=FALSE
-getConc=function(fit, y, n.replicate=1, fit.4pl=FALSE, incr=TRUE, check.out.of.range=FALSE, x.range=NULL, y.range=NULL, verbose=FALSE) {
+
+# n.replicate=1; check.out.of.range=F; x.range=NULL; y.range=NULL; verbose=FALSE
+getConc=function(fit, y, n.replicate=1, check.out.of.range=TRUE, x.range=NULL, y.range=NULL, verbose=FALSE) {
     
     if (verbose) print("getConc 100")
     param=coef(fit,type="classical")
     names(param)=substr(names(param),1,1)
-    b=param["b"]; c=param["c"]; d=param["d"]; e=param["e"]; f=param["f"]; if(fit.4pl==TRUE) f=1
-    
-    # fill-in value when out of bound
-    if (incr) {
-        x.inf = (x.range[2])
-        x.ninf = (x.range[1]/2)        
+    b=param["b"]; c=param["c"]; d=param["d"]; e=param["e"]
+    if (is.na(param["f"])) {
+        fit.4pl=TRUE
+        f=1
     } else {
-        x.inf = (x.range[1])
-        x.ninf = (x.range[2]/2)
+        fit.4pl=FALSE
+        f=param["f"]
     }    
-    
+        
     if (class(fit)=="drc") {
                     
+        if (verbose) print("getConc 200")
+        
         # point estimate
-        x = FivePL.x.inv(y, param, incr) 
+        x = FivePL.x.inv(y, param) 
                     
         # variance part 1
         A=((d-c)/(y-c))^(1/f)-1
@@ -397,8 +409,9 @@ getConc=function(fit, y, n.replicate=1, fit.4pl=FALSE, incr=TRUE, check.out.of.r
         sigma2 = getVarComponent(fit)
         s1=dx.dy^2*sigma2/n.replicate /x^2
         
+        if (verbose) print("getConc 300")
         # variance part 2
-        dx.db = -log(A)*A^(1/b)*b^(-2)*e
+        dx.db = suppressWarnings(-log(A)*A^(1/b)*b^(-2)*e) # suppressWarnings to avoid NaN warning
         dx.dc = e/b*A^(1/b-1)*B^(1/f-1) *(1/f)*(d-y)/(y-c)^2
         dx.dd = e/b*A^(1/b-1)*B^(1/f-1) *(1/f)/(y-c)
         dx.de = A^(1/b)
@@ -413,20 +426,19 @@ getConc=function(fit, y, n.replicate=1, fit.4pl=FALSE, incr=TRUE, check.out.of.r
 #        print(D.beta%*%V.beta) # from jags output, the correlation between e and f is very different from drm output. 
 #        print(s2)
     
+        if (verbose) print("getConc 400")
         se.t = sqrt(s1 + s2)
         lower.bound=suppressWarnings(exp(log(x)-2*se.t))
         upper.bound=suppressWarnings(exp(log(x)+2*se.t))
-        
+                
     } else if (class(fit)=="bcrm") {
     
         if (verbose) print("getConc 500")
         params=fit$coef.samples
-        x.samples = FivePL.x.inv(y=rep(y,each=nrow(params)), param=rep.matrix(params, times=length(y), by.row=TRUE), incr) 
+        x.samples = FivePL.x.inv(y=rep(y,each=nrow(params)), param=rep.matrix(params, times=length(y), by.row=TRUE)) 
         x.samples=matrix(x.samples,nrow=nrow(params))
         
         # need to do this here before computing s2, b/c if there is Inf in x.samples, s2 will be NaN
-        x.samples[x.samples==Inf]=x.inf
-        x.samples[x.samples==-Inf]=x.ninf
         
         x=apply(x.samples,2,median)        
         
@@ -453,11 +465,18 @@ getConc=function(fit, y, n.replicate=1, fit.4pl=FALSE, incr=TRUE, check.out.of.r
         upper.bound=upper.bound-1.96*sqrt(s1)
         upper.bound=upper.bound+1.96*sqrt(s1)
         upper.bound=exp(upper.bound)
+    
+        x=apply(x.samples,2,median)        
     }
     
-    # if estimated conc outside estimated asymptote, set to Inf
     if (verbose) print("getConc 800")
-    x=ifelse(y<c, x.ninf, ifelse(y>d, x.inf, x) )
+    if (check.out.of.range) {
+        if (!is.null(x.range)) {
+            x = treat.out.of.bound.2 (x, x.range)
+        }
+    }
+    
+    # if estimated conc outside estimated asymptote, set se to Inf
     se1 = Inf
     se.t=ifelse(y<c | y>d, se1, se.t)
     
@@ -482,6 +501,8 @@ getConc=function(fit, y, n.replicate=1, fit.4pl=FALSE, incr=TRUE, check.out.of.r
 #        }
 #    }
     
+    
+    #need to fix log x warning
     if (verbose) print("getConc 900")
     res=drop(cbind(
         "log.conc"=unname(log(x)), 
